package beans;

import data.DataSet;
import data.Recipient;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;
import java.util.stream.Collectors;
import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.faces.view.ViewScoped;
import javax.inject.Inject;
import javax.inject.Named;
import javax.servlet.ServletContext;
import javax.servlet.http.Part;
import javax.ws.rs.ClientErrorException;
import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.Entity;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import org.apache.commons.io.FileUtils;
import org.apache.poi.EncryptedDocumentException;
import org.apache.poi.ss.usermodel.DataFormatter;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;

@ViewScoped
@Named
public class Multimail implements Serializable {

    private boolean visible=false;      // INVボタンの表示・非表示
    private String sender;              // 送信者メール
    private String title;               // メールタイトル
    private String message;             // 本文 or HTML文
    private List<Recipient> dlist
              = new ArrayList<>();      // 宛先のリスト
    private boolean toggle = false;     // 全対象者のON／OFF
    private Recipient item;             // 宛先リストの項目名
    private JmClient jmClient;          // RESTクライアント

    private String dir;     // ファイルを格納するディレクトリ名
    private Part file;      // Mime形式のアップロードされたファイル
    private String fname;   // アップロードされたファイル名

    @Inject
    ServletContext context;
    @Inject
    transient Logger log;

    @PostConstruct
    public void init() {
        jmClient = new JmClient();                          // RESTクライアントを作成
        dir = context.getRealPath("resources") + "/files";  // Excelファイルを置くフォルダ
        createFileDir();                                    // dirフォルダがなければ作成する
    }
    // resources/filesフォルダ確認（作成）・フォルダ内をクリアする
    public void createFileDir() {
        try {
            Path dirPath = Paths.get(dir);
            if (!Files.exists(dirPath)) {
                Files.createDirectory(dirPath);
            }
            FileUtils.cleanDirectory(new File(dir)); // ディレクトリ内をクリアしておく
        } catch (IOException ex) {
            log.severe(ex.toString());
        }
    }
    // selectedをすべてtrue/falseに変更する（トグル処理）
    public void reverse(){
        dlist.stream().forEach(d->d.setFlag(toggle));
        toggle = !toggle;
    }
    // RESTクライアントを使って複数メールを送信する
    public void sendmails() {
        if(!dataCheck()) return;
        List<Recipient> ls = dlist.stream()
                            .filter(Recipient::isFlag)      
                            .collect(Collectors.toList());
        if(ls.isEmpty()) return; // 送信対象がない
        
        DataSet ds = new DataSet(sender, title, message, "TO", ls);
        Response res = jmClient.send(ds);
        clear();    // 表示をクリア
        file=null;  // アップロードファイルもクリアする
        message = res.readEntity(String.class); // 応答を取得
    }
    // 送信データが入力されたかどうか
    public boolean dataCheck(){
        if(sender==null || sender.isBlank()) return false;
        if(title==null || title.isBlank()) return false;
        if(message==null || message.isBlank()) return false;
        if(dlist.isEmpty()) return false; // ファイルが指定されていない
        return true;
    } 
    // 表示用の変数をクリアする
    public void clear(){
        toggle = false; // チェックボックスのON/OFF
        visible=false;  // form3の表の項目名の表示・非表示
        title=null;
        message=null;
        dlist.clear();
    }
    public void clearAll(){
        clear();
        file=null;
    }
    // アップロードファイルをフォルダ（dir）に受け取り
    // Excelファイルから宛先リストを作成する
    public void upload() {
        if(file==null)  return; // アップロードをキャンセルした時の対応
        clear();
        fname = file.getSubmittedFileName();    // 受信したファイルの名前
        try(InputStream in = file.getInputStream();){
            FileUtils.copyInputStreamToFile(in, new File(dir, fname));
            visible = true; // ボタンを表示する
        } catch (IOException ex) {
            log.severe(ex.toString());
        }
        getRecipientList(); // dlistを作成する
    }
    // 宛先オブジェクトのリストを作成する
    public void getRecipientList() {
        Sheet sheet = sheet();
        DataFormatter dataFormatter = new DataFormatter();  // セルの値をStringにフォーマット
        for (Row row : sheet) {
            int max = row.getLastCellNum();
            if(max <3) continue;    // 1オリジン
            Recipient recipient = new Recipient(
                true,                                          // 状態：送信対象者はtrue     
                dataFormatter.formatCellValue(row.getCell(0)),                  // ID
                dataFormatter.formatCellValue(row.getCell(1)),                  // 氏名
                dataFormatter.formatCellValue(row.getCell(2)),                  // メール
                max>3 ? dataFormatter.formatCellValue(row.getCell(3)) : "様"    // 敬称
            );
            dlist.add(toUtf8(recipient));// 日本語部分をシステムの文字コードに変換して加える
        }
        item = dlist.remove(0); // 先頭は項目名なので削除し、項目見出しに使う
    }
    // 宛先に含まれる日本語をシステムのデフォルトにエンコードする
    public Recipient toUtf8(Recipient r){
        r.setName(new String(r.getName().getBytes()));  
        r.setMr(new String(r.getMr().getBytes()));        
        return r;
    }
    // ワークブックから先頭のシートを取り出して返す
    public Sheet sheet() {
        Workbook workbook = null;
        try {
            workbook = WorkbookFactory.create(new File(dir + "/" + fname));
        } catch (IOException | EncryptedDocumentException ex) {
            log.severe(ex.toString());
        }
        return workbook.getSheetAt(0); // 先頭のシート
    }    
    // REST クライアント
    static class JmClient {
        private WebTarget webTarget;
        private Client client;
        private static final String BASE_URI = "http://localhost:8080/mailservice/sendmail";

        public JmClient() {
            client = ClientBuilder.newClient();
            webTarget = client.target(BASE_URI).path("cc");
        }
        public Response send(DataSet ds) throws ClientErrorException {
            return webTarget.path("text").request()
                .post(Entity.entity(ds, MediaType.APPLICATION_JSON));
        }
        @PreDestroy
        public void close() {
            client.close();
        }
    }
    // セッターとゲッター
    public Part getFile() {
        return file;
    }
    public void setFile(Part file) {
        this.file = file;
    }
    public String getFname() {
        return fname;
    }
    public void setFname(String fname) {
        this.fname = fname;
    }
    public boolean isVisible() {
        return visible;
    }
    public void setVisible(boolean visible) {
        this.visible = visible;
    }
    public String getSender() {
        return sender;
    }
    public void setSender(String sender) {
        this.sender = sender;
    }
    public String getTitle() {
        return title;
    }
    public void setTitle(String title) {
        this.title = title;
    }
    public String getMessage() {
        return message;
    }
    public void setMessage(String message) {
        this.message = message;
    }
    public List<Recipient> getDlist() {
        return dlist;
    }
    public void setDlist(List<Recipient> dlist) {
        this.dlist = dlist;
    }
    public Recipient getItem() {
        return item;
    }
    public void setItem(Recipient item) {
        this.item = item;
    }
}
