package app;
/*
 * 電子メール送信サービス 
 */
import java.util.List;
import java.util.Properties;
import java.util.concurrent.CompletableFuture;
import java.util.logging.Logger;
import java.util.stream.Collectors;
import javax.annotation.PostConstruct;
import javax.enterprise.context.RequestScoped;
import javax.inject.Inject;
import javax.ws.rs.Consumes;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.Status;
import jmail.EmailSender;
import jmail.EmailStatus;

@RequestScoped
@Path("cc")
@Produces({MediaType.APPLICATION_JSON, MediaType.APPLICATION_XML})
@Consumes({MediaType.APPLICATION_JSON, MediaType.APPLICATION_XML})
public class Sender {
    
    @Inject private EmailSender es;
    @Inject transient private Logger log;
    /**
     * 同報・平文メール送信（コンカレント処理）
     * @param data 送信データオブジェクト
     * @return 通知文を返す
     */    
    @Path("/text")
    @POST
    public Response send(DataSet data) {
        if(DataSet.checkData(data)!=0){
            return Response.status(Status.BAD_REQUEST)
                           .entity("送信されたデータに誤りがあります。").build();
        }
       CompletableFuture.supplyAsync(()->sendMails(data))
                        .thenAccept(result->reply(data,result));
        return Response
                .ok("送信を開始します。この後、送信結果のメールが届きます。")
                .build();
    }
    // 平文・複数メールの送信
    private EmailStatus sendMails(DataSet data){
        // 接続
        EmailStatus sts = es.connect();                                                
        if(sts!=EmailStatus.DONE)   return  sts;
        // 送信
        List<Recipient> rs = data.getDlist(); // 宛先リストを取り出す
        for(Recipient r : rs){                                     
            StringBuilder sb = new StringBuilder();
            sb.append(r.getName()).append(" ")  // 宛名
              .append(r.getMr()).append("\n")   // 敬称
              .append(data.getMessage());       // メール本文

            sts = es.xsend(r.getEmail(), data.getTitle(), sb.toString()); // メール送信
            if(sts!=EmailStatus.DONE) r.setFlag(false); // 未送信の宛先にはfalseを付けておく
        }
        // 切断
        sts = es.disconnect();                                                         
        return sts;
    }
    // 送信結果通知メールの送信
    private void reply(DataSet data, EmailStatus status){
        
        String result = returnMessage(status); // メッセージに変換
        int all = data.getDlist().size(); // 全件数
        int done = doneMails(data);       // 送信件数
        int errs = all - done;            // エラー件数
        String errors = errMails(data);   // エラーアドレス（CSV）
        
        System.out.println("★all =" + all);
        System.out.println("★done=" + done);
        System.out.println("★errs=" + errs);
        System.out.println("★errors=" + errors);
        
        
        StringBuilder sb = new StringBuilder();
        sb.append(result).append("\n\n")
          .append("\t受付件数　").append(all).append("件\n")
          .append("\t送信件数　").append(done).append("件\n");
        if(errs!=0){
            sb.append("\n送信不能　").append(errs).append("件\n")
            .append("\t次の宛先は無効です\n\t").append(errors);
        }
        sb.append("\n--------\n").append(data.getMessage());

        es.send(data.getSender(), "送信結果のお知らせ", sb.toString());
    }
    // 送信済件数を返す
    public int doneMails(DataSet data){
        int n = (int)data.getDlist().stream()
                        .filter(Recipient::isFlag)  // flagがtrueのものだけ
                        .count();                   // 件数を得る
        return n;
    }
    // 送信エラーのアドレスをCSV文字列にして返す
    public String errMails(DataSet data){
        String errors = data.getDlist().stream()
                            .filter(r->!r.isFlag())   // flagがfalseのものだけ
                            .map(Recipient::summary) // 情報を要約する
                            .collect(Collectors.joining("\n")); // 改行で連結する 
        return errors;
    }
    // 終了コードをメッセージに変換して返す
    private String returnMessage(EmailStatus status){
        switch(status){
            case INIT_ERROR:        return "ユーザー名・パスワードが設定されていません";
            case CONNECT_ERROR:     return "SMTPサーバーに接続できないため送信できませんでした";
            case SEND_ERROR:        return "送信中に予期しないエラーが発生したため送信できませんでした";
            case DISCONNECT_ERROR:  return "切断時のエラーのため送信できませんでした";
            default:                return "送信完了しました";
        }
    }
}